^IAL^
*
* 8008 Inverse Assembler for HP logic analyzers
*
* This version reads the addresses from the latches
* and does not require that each analyzer state equal one CPU state.
*
* ADDR label - connect to address latches
* DATA label - connect to 8008 D7-D0 (bit 7 to pin 2 .. bit 0 to pin 9)
* STAT label - connect as follows:
*	8008 S0 (pin 13)	bit 2
*	8008 S1	(pin 12)	bit 1
*	8008 S2 (pin 11)	bit 0
* trigger on falling edge of ST3*SY or similar
*
* N.b. voltage levels are 3.5V on 8008 pins
*

        SEARCH_LIMIT    5
        DEFAULT_WIDTH   25
        MAPPED_WIDTH    25
        LABEL_TITLE     ^8008 Mnemonic^
        BASE_TITLE      ^     Octal^

T3      CONSTANT        100B
T4      CONSTANT        111B

LOA     VARIABLE
BASE    VARIABLE

A_FMT   FORMAT  14,OCT,5
D_FMT   FORMAT  8,OCT,3
DEC_FMT FORMAT  3,DEC,1

        SET     RETURN_FLAGS,0
        CALL    RESET_QUAL

        IF INPUT_ERROR = 0 THEN GOTO ANALYZE
        OUTPUT  "Data Error"
        ABORT
ANALYZE LOAD INPUT_STATUS
        CASE_OF 2,0
          OUTPUT "  (WAIT)"
          NOP
          NOP
          CALL ST1I
          CALL ST3
          NOP
          OUTPUT "  (STOP)"
          NOP
        CASE_END
        RETURN

RESET_QUAL
        SET     QUALIFY_MASK,111B       * S0-S2
        SET     QUALIFY_VALUE,100B      * T3
        RETURN

ST1I    OUTPUT  " Int. Ack."
        RETURN

ST3     LOAD    INPUT_TAG
        IF 0,0 = 1 THEN RETURN

        LOAD    INPUT_DATA
        CASE_OF 7,6
          CALL UD0
          CALL UD1
          CALL UD2
          CALL UD3
        CASE_END
        RETURN

UD0     IF 5,1 = 00000B THEN GOTO HLT
        CASE_OF 2,0
          CALL INr
          CALL DCr
          CALL Shifts
          CALL Rets
          CALL AluI
          CALL RSTs
          CALL LrIs
          CALL RET
        CASE_END
        RETURN

INr     OUTPUT "IN"
        GOTO REG53        

DCr     OUTPUT "DC"
        GOTO REG53

Shifts  CASE_OF 5,3
          OUTPUT "RLC"
          OUTPUT "RRC"
          OUTPUT "RAL"
          OUTPUT "RAR"
        CASE_END
        RETURN

Rets    OUTPUT "R"
CONDCODE
        CASE_OF 5,3
          OUTPUT "FC"
          OUTPUT "FZ"
          OUTPUT "FS"
          OUTPUT "FP"
          OUTPUT "FC"
          OUTPUT "TZ"
          OUTPUT "TS"
          OUTPUT "TP"
        CASE_END
        RETURN

AluI    CASE_OF 5,3
          OUTPUT "ADI"
          OUTPUT "ACI"
          OUTPUT "SUI"
          OUTPUT "SBI"
          OUTPUT "NDI"
          OUTPUT "XRI"
          OUTPUT "ORI"
          OUTPUT "CPI"
        CASE_END
IMMCYCLE
        OUTPUT " "
        INPUT REL,1,QUALIFIED
        LOAD INPUT_DATA
        OUTPUT ACCUMULATOR,D_FMT
        TAG_WITH 1
        RETURN

RSTs    OUTPUT "RST "
        ROTATE RIGHT,3
        OUTPUT ACCUMULATOR,DEC_FMT
        RETURN

LrIs    OUTPUT "L"
        CALL REG53
        OUTPUT "I"
        IF 5,3 = 111B THEN GOTO LMI
        GOTO IMMCYCLE

LMI     CALL IMMCYCLE
        LOAD 2
MEMCYCLE STORE BASE
        INPUT REL,BASE,QUALIFIED
        LOAD INPUT_ADDRESS
        OUTPUT " ["
        IF_NOT_MAPPED THEN OUTPUT ACCUMULATOR,A_FMT
        OUTPUT "]="
        LOAD INPUT_DATA
        OUTPUT ACCUMULATOR,D_FMT
        TAG_WITH 1
        RETURN

RET     OUTPUT "RET"
        RETURN

UD1     IF 0,0 = 1 THEN GOTO IOs
        CASE_OF 2,1
          OUTPUT "J"
          OUTPUT "C"
          OUTPUT "JMP"
          OUTPUT "CAL"
        CASE_END
        IF 2,2 = 0 THEN CALL CONDCODE
        OUTPUT " "
        INPUT REL,1,QUALIFIED
        LOAD INPUT_DATA
        STORE LOA
        TAG_WITH 1
        INPUT REL,2,QUALIFIED
        LOAD INPUT_DATA
        TAG_WITH 1
        ROTATE LEFT,8
        INCLUSIVE_OR LOA
        IF_NOT_MAPPED THEN OUTPUT ACCUMULATOR,A_FMT
        RETURN

IOs     CASE_OF 5,4
          OUTPUT "INP"
          OUTPUT "OUT"
          OUTPUT "OUT"
          OUTPUT "OUT"
        CASE_END
        OUTPUT " "
        ROTATE RIGHT,1
        AND 11111B
        OUTPUT ACCUMULATOR,D_FMT

        QUALIFY_VALUE T4
        INPUT REL,0,QUALIFIED
        IF INPUT_ERROR <> 0 THEN GOTO NO_A 
        OUTPUT " A="
        LOAD INPUT_DATA
        OUTPUT ACCUMULATOR,D_FMT
NO_A    CALL RESET_QUAL
        INPUT REL,1,QUALIFIED
        TAG_WITH 1
        LOAD INPUT_DATA
        OUTPUT " I/O="
        OUTPUT ACCUMULATOR,D_FMT
        RETURN

UD2     CASE_OF 5,3
          OUTPUT "AD"
          OUTPUT "AC"
          OUTPUT "SU"
          OUTPUT "SB"
          OUTPUT "ND"
          OUTPUT "XR"
          OUTPUT "OR"
          OUTPUT "CP"
        CASE_END
        CALL    REG20
        IF 2,0 <> 111B THEN RETURN
        LOAD 1
        GOTO MEMCYCLE

HLT     OUTPUT "HLT"
        RETURN

UD3     IF 5,0 = 111111B THEN GOTO HLT
        OUTPUT "L"
        CALL REG53
        CALL REG20
        IF 5,3 = 7 THEN GOTO LMr
        IF 2,0 = 7 THEN GOTO LrM
        QUALIFY_VALUE T4
        INPUT REL,0,QUALIFIED
        IF INPUT_ERROR <> 0 THEN GOTO NO_VAL
        OUTPUT " "      * reg value on bus; show it
        CALL REG20
        OUTPUT "="
        LOAD INPUT_DATA
        OUTPUT ACCUMULATOR,D_FMT
NO_VAL  RETURN

LrM     INPUT REL,1,QUALIFIED
        GOTO Lrr
LMr     INPUT REL,1,QUALIFIED
Lrr     TAG_WITH 1
        OUTPUT " = "
        LOAD INPUT_DATA
        OUTPUT ACCUMULATOR,D_FMT
        RETURN

REG53   CASE_OF 5,3
          OUTPUT "A"
          OUTPUT "B"
          OUTPUT "C"
          OUTPUT "D"
          OUTPUT "E"
          OUTPUT "H"
          OUTPUT "L"
          OUTPUT "M"
        CASE_END
        RETURN

REG20   CASE_OF 2,0
          OUTPUT "A"
          OUTPUT "B"
          OUTPUT "C"
          OUTPUT "D"
          OUTPUT "E"
          OUTPUT "H"
          OUTPUT "L"
          OUTPUT "M"
        CASE_END
        RETURN

